/***********************/
/* viewlace.c 1.00     */
/* for TestLace        */
/* by Adam M. Costello */
/* (amc@cs.wustl.edu)  */
/***********************/

/* This is ANSI C plus X11R5 code. */


#include "fail.h"
#include "lacescheme.h"
#include "preview.h"
#include "visit.h"
#include "timer.h"
#include "window.h"

#include <stdio.h>
#include <stdlib.h>
#include <X11/Xlib.h>


void usage(void)
/* Prints a usage message to stderr, then exits unsuccessfully. */
{
  fputs(
    "Options:\n"
    "  -display <displayname>   Choose X display\n"
    "  -name    <winname>       Set window name (default is argv[0])\n"
    "  -f       <frequency>     Set frequency in pixels/s (default is 20000)\n"
    "  -pr      <prfilename>    Show a preview before the interlace scheme\n"
    "                             (uses aux file created by makepreview)\n"
    , stderr);

  laceopts(stderr);

  fail(
    "           <ppmfilename>   Read image from a file (default is stdin)\n"
    "\n"
    "Note:  Input and preview images must be rawbit PPM files (P6) with\n"
    "       maxval = 255.  They must use no more than 256 colors combined.\n"
    "       The X display must be capable of 8-bit PseudoColor.\n"
    "\n"
    );
}


void parseargs(
  int argc, const char **argv, const char **displayname,
  const char **winname, int *freq, schemeimpl *dolace,
  const char **prfilename, const char **ppmfilename
)
/* Parses the command line arguments and sets *displayname, *winname, */
/* *dolace, prfilename, and *ppmfilename if they are specified on the */
/* command line.  Does not modify variables whose options are not     */
/* given.  Calls fail() if the arguments are improper.                */
{
  schemeimpl tmp;

  while (*++argv) {
    if        (!strcmp(*argv, "-display")) {
      if (!*++argv) usage();
      *displayname = *argv;
    } else if (!strcmp(*argv, "-name")) {
      if (!*++argv) usage();
      *winname = *argv;
    } else if (!strcmp(*argv, "-f")) {
      if (!*++argv) usage();
      *freq = atoi(*argv);
    } else if (!strcmp(*argv, "-pr")) {
      if (!*++argv) usage();
      *prfilename = *argv;
    } else {
      tmp = lacescheme(*argv);
      if (tmp) {
        *dolace = tmp;
      } else if (**argv != '-') {
        *ppmfilename = *argv;
        if (argv[1]) usage();
      } else {
        usage();
      }
    }
  }
}


void eventloop(void)
/* Just waits for a button 1 press, then returns. */
{
  XEvent event, *ep = &event;

  do XNextEvent(display,ep);
  while (event.type != ButtonPress || event.xbutton.button != Button1);
}


int main(int argc, const char **argv)
{
  int freq = 20000, prwidth, prheight, prmaxval, width, height, maxval;
  const char *displayname = NULL, *winname = argv[0], *prfilename = NULL,
             *ppmfilename = NULL;
  schemeimpl dolace = defaultscheme;
  FILE *ppmfile = NULL, *prfile = NULL;
  IMAGE *image = NULL, *primage = NULL;

  parseargs(argc, argv, &displayname, &winname,
            &freq, &dolace, &prfilename, &ppmfilename);

  if (!ppmfilename) ppmfile = stdin;
  else {
    ppmfile = fopen(ppmfilename, "rb");
    if (!ppmfile) fail("Cannot open input file.\n");
  }

  if (fgetc(ppmfile) != 'P' || fgetc(ppmfile) != '6')
    fail("Input is not a P6 file.\n");

  fscanf(ppmfile, "%d %d %d", &width, &height, &maxval);
  if (maxval != 255) fail("Input has maxval != 255\n");
  fgetc(ppmfile);

  if (prfilename) {
    prfile = fopen(prfilename, "rb");
    if (!prfile) fail("Cannot open preview file.\n");
    if (fgetc(prfile) != 'P' || fgetc(prfile) != '6')
      fail("Preview is not a P6 file.\n");
    fscanf(prfile, "%d %d %d", &prwidth, &prheight, &prmaxval);
    if (prmaxval != 255) fail("Preview has maxval != 255\n");
    if (prwidth != width || prheight != height)
      fail("Preview has different dimensions than input.\n");
    fgetc(prfile);
  }

  warn("Preloading image...\n");
  image = loadimage(ppmfile,width,height);
  close(ppmfile);
  ppmfile = NULL;

  if (prfile) {
    warn("Preloading preview...\n");
    primage = loadimage(prfile,width,height);
    close(prfile);
    prfile = NULL;
  }

  display = XOpenDisplay(displayname);
  if (!display) fail("Cannot open display.\n");

  initwindow(winname,width,height);

  settimer(freq);

  if (primage) {
    useimage(primage);
    preview(width,height);
  }

  useimage(image);
  dolace(width,height);

  XFlush(display);
  puts("Done.  Click on window to exit.");
  fflush(stdout);

  eventloop();

  return 0;
}
