/***********************/
/* window.c 1.01       */
/* for TestLace        */
/* by Adam M. Costello */
/* (amc@cs.wustl.edu)  */
/***********************/

/* This is ANSI C plus X11R5 code. */


#include "window.h"  /* Also includes <X11/Xlib.h>. */
#include "fail.h"

#include <string.h>
#include <X11/Xutil.h>


enum {depth = 8};

static const char *const
  cantopenwin = "Can't open window.\n";  /* Error message. */

static int width, height;

static char windowexists, gcexists, colormapexists;
  /* The above are initialized to 0. */
static Window window;
static GC gc;
static Colormap colormap;


static void cleanup(void)
{
  if (gcexists) XFreeGC(display,gc);
  if (windowexists) XDestroyWindow(display,window);
  if (colormapexists) XFreeColormap(display,colormap);
}


void initwindow(const char *name, int w, int h)
{
  int screen;
  Window root;
  XVisualInfo visual_info;
  Visual *visual;
  XSetWindowAttributes attributes;
  unsigned long valuemask;
  XSizeHints sizehints;
  XWMHints wmhints;
  XGCValues gcvalues;
  XEvent event, *ep = &event;

  atexit(cleanup);

  width  = w;
  height = h;

  screen = DefaultScreen(display);
  root   = RootWindow(display,screen);

  if (!XMatchVisualInfo(display, screen, depth, PseudoColor, &visual_info))
    fail("Screen does not support PseudoColor visual of desired depth.\n");
  visual = visual_info.visual;

  colormap = XCreateColormap(display,root,visual,AllocNone);
  if (!colormap) fail("Cannot create colormap.\n");
  colormapexists = 1;

  valuemask = CWBackingStore | CWEventMask;

  attributes.backing_store    = Always;
  attributes.event_mask       = ExposureMask | ButtonPressMask;

  window = XCreateWindow(display, root, 0, 0, width, height, 0, depth,
                         InputOutput, visual, valuemask, &attributes);
  if (window == None) fail(cantopenwin);
  windowexists = 1;

  XSetWindowColormap(display,window,colormap);

  sizehints.flags = PMinSize | PMaxSize | PBaseSize;
  sizehints.min_width  = sizehints.max_width  = sizehints.base_width  = width;
  sizehints.min_height = sizehints.max_height = sizehints.base_height = height;
  XSetWMNormalHints(display, window, &sizehints);

  wmhints.flags = InputHint | StateHint;
  wmhints.input = True;
  wmhints.initial_state = NormalState;
  XSetWMHints(display, window, &wmhints);

  XStoreName(display,window,name);

  XMapWindow(display,window);

  valuemask = 0;
  gc = XCreateGC(display, window, valuemask, &gcvalues);
  gcexists = 1;

  do XNextEvent(display,ep);
  while (   ep->xany.window != window
         || ep->type != Expose || ep->xexpose.count);

  XFlush(display);
}


void drawrect(int x, int y, int w, int h, unsigned short red,
                unsigned short green, unsigned short blue)
{
  static unsigned long numcolors = 0, pixels[1 << depth];
  static unsigned short reds[1 << depth], greens[1 << depth],
                  blues[1 << depth];
  unsigned long i;
  XColor xcolor;

  for (i = 0;  i < numcolors;  ++i)
    if (red == reds[i] && green == greens[i] && blue == blues[i]) break;

  if (i == numcolors) {
    if (numcolors == 1 << depth) fail("Image has too many colors.\n");
    ++numcolors;
    reds[i]   = red;
    greens[i] = green;
    blues[i]  = blue;
    xcolor.red   = red;
    xcolor.green = green;
    xcolor.blue  = blue;
    xcolor.flags = DoRed | DoGreen | DoBlue;
    if (!XAllocColor(display, colormap, &xcolor))
      fail("Unable to allocate colormap entry.\n");
    pixels[i] = xcolor.pixel;
    if (i == 0) XFillRectangle(display,window,gc,0,0,width,height);
  }

  XSetForeground(display, gc, pixels[i]);
  XFillRectangle(display,window,gc,x,y,w,h);
}


Display *display;
